/* ++++++++++
	driver.c
	A skeletal device driver
+++++ */

#include <KernelExport.h>
#include <Drivers.h>
#include <bone/sys/socket_module.h>

#define DRIVER_NAME "ksock"

static bone_socket_info_t *mod_bone;


int send_udp()
{
	int s, size;
	struct sockaddr_in sin;
	struct sockaddr_in toaddr;
	
	const char *data = "12345";	// 送信したいデータ
	
	memset(&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	
	s = mod_bone->socket(AF_INET, SOCK_DGRAM, 0);
	if (s < 0) {
		dprintf(DRIVER_NAME ": socket error: %s\n", strerror(errno));
		return 1;
	}
	
	toaddr.sin_family = AF_INET;
	toaddr.sin_port = htons(4989);	// 送信先ポート
	toaddr.sin_addr.s_addr = mod_bone->inet_addr("192.168.0.5");	// 送信先アドレス
	
	size = mod_bone->sendto(s, data, strlen(data), 0, (struct sockaddr *)&toaddr, sizeof(toaddr));
	if ( size < 0) {
		dprintf(DRIVER_NAME ": sendto error: %s\n", strerror(errno));
		if (close(s) != 0) {
			dprintf(DRIVER_NAME ": close error: %s\n", strerror(errno));
		}
		return 1;
	}
	dprintf(DRIVER_NAME ": %d send\n", size);
	
	if (close(s) != 0) {
		dprintf(DRIVER_NAME ": close error: %s\n", strerror(errno));
		return 1;
	}
	
	return 0;
}

/* ----------
	init_hardware - called once the first time the driver is loaded
----- */
status_t
init_hardware (void)
{
	set_dprintf_enabled(true);
	dprintf(DRIVER_NAME ": init_hardware\n");
	
	return B_OK;
}


/* ----------
	init_driver - optional function - called every time the driver
	is loaded.
----- */
status_t
init_driver (void)
{
	dprintf(DRIVER_NAME ": init_driver\n");
	
	if (get_module(BONE_SOCKET_MODULE, (module_info **)&mod_bone)) {
		dprintf(DRIVER_NAME ": get_module failed\n");
		return ENOSYS;
	}
	
	return B_OK;
}


/* ----------
	uninit_driver - optional function - called every time the driver
	is unloaded
----- */
void
uninit_driver (void)
{
	dprintf(DRIVER_NAME ": uninit_driver\n");
	put_module(BONE_SOCKET_MODULE);
}

	
/* ----------
	my_device_open - handle open() calls
----- */

static status_t
my_device_open (const char *name, uint32 flags, void** cookie)
{
	dprintf(DRIVER_NAME ": my_device_open\n");
	send_udp();
	return B_OK;
}


/* ----------
	my_device_read - handle read() calls
----- */

static status_t
my_device_read (void* cookie, off_t position, void *buf, size_t* num_bytes)
{
	dprintf(DRIVER_NAME ": my_device_open\n");
	*num_bytes = 0;				/* tell caller nothing was read */
	return B_IO_ERROR;
}


/* ----------
	my_device_write - handle write() calls
----- */

static status_t
my_device_write (void* cookie, off_t position, const void* buffer, size_t* num_bytes)
{
	dprintf(DRIVER_NAME ": my_device_write\n");
	*num_bytes = 0;				/* tell caller nothing was written */
	return B_IO_ERROR;
}


/* ----------
	my_device_control - handle ioctl calls
----- */

static status_t
my_device_control (void* cookie, uint32 op, void* arg, size_t len)
{
	dprintf(DRIVER_NAME ": my_device_control\n");
	return B_BAD_VALUE;
}


/* ----------
	my_device_close - handle close() calls
----- */

static status_t
my_device_close (void* cookie)
{
	dprintf(DRIVER_NAME ": my_device_close\n");
	return B_OK;
}


/* -----
	my_device_free - called after the last device is closed, and after
	all i/o is complete.
----- */
static status_t
my_device_free (void* cookie)
{
	dprintf(DRIVER_NAME ": my_device_free\n");
	return B_OK;
}


/* -----
	null-terminated array of device names supported by this driver
----- */

static const char *my_device_name[] = {
	"misc/ksock",
	NULL
};

/* -----
	function pointers for the device hooks entry points
----- */

device_hooks my_device_hooks = {
	my_device_open, 			/* -> open entry point */
	my_device_close, 			/* -> close entry point */
	my_device_free,			/* -> free cookie */
	my_device_control, 		/* -> control entry point */
	my_device_read,			/* -> read entry point */
	my_device_write			/* -> write entry point */
};

/* ----------
	publish_devices - return a null-terminated array of devices
	supported by this driver.
----- */

const char**
publish_devices()
{
	dprintf(DRIVER_NAME ": publish_devices\n");
	return my_device_name;
}

/* ----------
	find_device - return ptr to device hooks structure for a
	given device name
----- */

device_hooks*
find_device(const char* name)
{
	dprintf(DRIVER_NAME ": find_device(%s)\n", name);
	return &my_device_hooks;
}

